"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs = require("fs-extra");
const os = require("os");
const path = require("path");
function describe() {
    return {
        code: 'exec-ifcopenshell',
        name: 'Execute IfcOpenShell to convert IFC files into Collada files',
        priority: 11,
    };
}
exports.describe = describe;
function shouldProcess(conv, context) {
    return context.assetsPaths &&
        context.assetsPaths.length &&
        context.assetsPaths.some(assetPath => assetPath.toLowerCase().endsWith('.ifc'));
}
exports.shouldProcess = shouldProcess;
function process(conv, context, progress) {
    return __awaiter(this, void 0, void 0, function* () {
        //=> Create a temporary folder for the assets
        const tmpDir = path.resolve(`${os.tmpdir()}/chuck/exec-ifcopenshell-${Date.now()}`);
        yield fs.mkdirp(tmpDir);
        context.convertedAssetsDir = tmpDir;
        //=> Execute each conversion sequentially
        // @todo If IfcConvert is mono-threaded, we could make it concurrent
        for (const assetPath of context.assetsPaths) {
            if (assetPath.toLowerCase().endsWith('.ifc')) {
                yield progress('convert-start', `Converting "${assetPath}" from IFC to Collada`);
                yield convertAndStoreAssets(context, assetPath);
            }
        }
    });
}
exports.process = process;
function cleanup(context) {
    return __awaiter(this, void 0, void 0, function* () {
        yield fs.remove(context.convertedAssetsDir);
    });
}
exports.cleanup = cleanup;
function convertAndStoreAssets(context, ifcFilePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const colladaFileName = path.parse(ifcFilePath).name + '.dae';
        const colladaFilePath = path.resolve(`${context.convertedAssetsDir}/${colladaFileName}`);
        const converterProcess = child_process_1.spawn('IfcConvert', [
            ifcFilePath, colladaFilePath,
            '-y', '--unicode', 'escape', '--use-element-hierarchy', '--use-element-types'
        ]);
        //=> Watch process' stdout to log in real time, and keep the complete output in case of crash
        let stdoutAggregator = '';
        converterProcess.stdout.on('data', data => stdoutAggregator += data.toString());
        converterProcess.stderr.on('data', data => stdoutAggregator += data.toString());
        //=> Watch for the process to terminate, check return code
        return new Promise((resolve, reject) => {
            converterProcess.once('close', (code) => {
                if (code !== 0) {
                    const message = `Conversion of IFC file ${path.basename(ifcFilePath)} to Collada has failed!`;
                    return reject(new IfcConvertCrashError(message, stdoutAggregator));
                }
                const index = context.assetsPaths.findIndex(path => path === ifcFilePath);
                context.assetsPaths[index] = colladaFilePath;
                resolve(colladaFilePath);
            });
        });
    });
}
class IfcConvertCrashError extends Error {
    constructor(message, ifcConvertLog) {
        super(message);
        this.ifcConvertLog = ifcConvertLog;
        Object.setPrototypeOf(this, IfcConvertCrashError.prototype);
    }
}
exports.IfcConvertCrashError = IfcConvertCrashError;
